@@ -38,10 +38,12 @@ typedef struct
 	int			fr;
 } SIMPLE_TIME;
 
-int		copy_and_convert_field_bindinfo(StatementClass *stmt, OID field_type, void *value, int col);
-int	copy_and_convert_field(StatementClass *stmt, OID field_type,
-			void *value, SQLSMALLINT fCType, PTR rgbValue,
-			SQLLEN cbValueMax, SQLLEN *pcbValue, SQLLEN *pIndicator);
+int	copy_and_convert_field_bindinfo(StatementClass *stmt, OID field_type, int atttypmod, void *value, int col);
+int	copy_and_convert_field(StatementClass *stmt,
+			OID field_type, int atttypmod,
+			void *value,
+			SQLSMALLINT fCType, int precision,
+			PTR rgbValue, SQLLEN cbValueMax, SQLLEN *pcbValue, SQLLEN *pIndicator);
 
 int		copy_statement_with_parameters(StatementClass *stmt, BOOL);
 BOOL		convert_money(const char *s, char *sout, size_t soutmax);
