@@ -13,6 +13,7 @@
  */
 
 #include "psqlodbc.h"
+#include "dlg_specific.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +36,8 @@
 #endif
 
 extern GLOBAL_VALUES globals;
-void	generate_filename(const char *, const char *, char *);
+
+static char *logdir = NULL;
 
 void
 generate_filename(const char *dirname, const char *prefix, char *filename)
@@ -168,12 +170,17 @@ mylog(const char *fmt,...)
 
 	if (!MLOGFP)
 	{
-		generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
+		generate_filename(logdir ? logdir : MYLOGDIR, MYLOGFILE, filebuf);
 		MLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (!MLOGFP)
 		{
 			generate_homefile(MYLOGFILE, filebuf);
 			MLOGFP = fopen(filebuf, PG_BINARY_A);
+			if (!MLOGFP)
+			{
+				generate_filename("C:\\podbclog", MYLOGFILE, filebuf);
+				MLOGFP = fopen(filebuf, PG_BINARY_A);
+			}
 		}
 		if (MLOGFP)
 			setbuf(MLOGFP, NULL);
@@ -217,7 +224,7 @@ forcelog(const char *fmt,...)
 
 	if (!MLOGFP)
 	{
-		generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
+		generate_filename(logdir ? logdir : MYLOGDIR, MYLOGFILE, filebuf);
 		MLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (MLOGFP)
 			setbuf(MLOGFP, NULL);
@@ -303,7 +310,7 @@ qlog(char *fmt,...)
 
 	if (!QLOGFP)
 	{
-		generate_filename(QLOGDIR, QLOGFILE, filebuf);
+		generate_filename(logdir ? logdir : QLOGDIR, QLOGFILE, filebuf);
 		QLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (!QLOGFP)
 		{
@@ -350,6 +357,11 @@ static void qlog_finalize() {}
 
 void InitializeLogging()
 {
+	char dir[PATH_MAX];
+
+	getLogDir(dir, sizeof(dir));
+	if (dir[0])
+		logdir = strdup(dir);
 	mylog_initialize();
 	qlog_initialize();
 }
@@ -358,4 +370,9 @@ void FinalizeLogging()
 {
 	mylog_finalize();
 	qlog_finalize();
+	if (logdir)
+	{
+		free(logdir);
+		logdir = NULL;
+	}
 }
