@@ -299,6 +299,14 @@ RETCODE SQL_API PGAPI_GetConnectAttr(HDBC ConnectionHandle,
 RETCODE SQL_API PGAPI_GetStmtAttr(HSTMT StatementHandle,
 		SQLINTEGER Attribute, PTR Value,
 		SQLINTEGER BufferLength, SQLINTEGER *StringLength);
+
+enum {
+	SQL_ATTR_PGOPT_DEBUG = 65536
+	,SQL_ATTR_PGOPT_COMMLOG
+	,SQL_ATTR_PGOPT_PARSE
+	,SQL_ATTR_PGOPT_USE_DECLAREFETCH
+	,SQL_ATTR_PGOPT_SERVER_SIDE_PREPARE
+};
 RETCODE SQL_API PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			SQLINTEGER Attribute, PTR Value,
 			SQLINTEGER StringLength);
